@@ -40,7 +40,6 @@ from sqlalchemy.orm import class_mapper
 from sqlalchemy.orm import ColumnProperty
 from sqlalchemy.orm import object_mapper
 from sqlalchemy.orm import RelationshipProperty
-from sqlalchemy.orm.dynamic import AppenderMixin
 from sqlalchemy.orm.exc import MultipleResultsFound
 from sqlalchemy.orm.exc import NoResultFound
 from sqlalchemy.orm.properties import RelationshipProperty as RelProperty
@@ -218,14 +217,20 @@ def _to_dict(instance, deep=None, exclude=None, include=None,
         elif (include_relations is not None and
               relation in include_relations):
             newinclude = include_relations[relation]
-        if isinstance(relatedvalue, list):
+        # Do some black magic on SQLAlchemy to decide if the related instance
+        # should be rendered as a list or as a single object.
+        uselist = instance._sa_class_manager[relation].property.uselist
+        if uselist:
             result[relation] = [_to_dict(inst, rdeep, exclude=newexclude,
                                          include=newinclude)
                                 for inst in relatedvalue]
-        else:
-            result[relation] = _to_dict(relatedvalue.one(), rdeep,
-                                        exclude=newexclude,
-                                        include=newinclude)
+            continue
+        # If the related value is dynamically loaded, resolve the query to get
+        # the single instance.
+        if isinstance(relatedvalue, Query):
+            relatedvalue = relatedvalue.one()
+        result[relation] = _to_dict(relatedvalue, rdeep, exclude=newexclude,
+                                    include=newinclude)
     return result
 
 
@@ -457,8 +462,8 @@ class API(ModelView):
     def __init__(self, session, model, authentication_required_for=None,
                  authentication_function=None, exclude_columns=None,
                  include_columns=None, validation_exceptions=None,
-                 results_per_page=10, post_form_preprocessor=None, *args,
-                 **kw):
+                 results_per_page=10, max_results_per_page=100,
+                 post_form_preprocessor=None, *args, **kw):
         """Instantiates this view with the specified attributes.
 
         `session` is the SQLAlchemy session in which all database transactions
@@ -508,10 +513,15 @@ class API(ModelView):
         See :ref:`includes` for information on specifying included or excluded
         columns on fields of related models.
 
-        `results_per_page` is a positive integer which represents the number of
-        results which are returned per page. If this is anything except a
-        positive integer, pagination will be disabled (warning: this may result
-        in large responses). For more information, see :ref:`pagination`.
+        `results_per_page` is a positive integer which represents the default
+        number of results which are returned per page. Requests made by clients
+        may override this default by specifying ``results_per_page`` as a query
+        argument. `max_results_per_page` is a positive integer which represents
+        the maximum number of results which are returned per page. This is a
+        "hard" upper bound in the sense that even if a client specifies that
+        greater than `max_results_per_page` should be returned, only
+        `max_results_per_page` results will be returned. For more information,
+        see :ref:`serverpagination`.
 
         `post_form_preprocessor` is a callback function which takes
         POST input parameters loaded from JSON and enhances them with other
@@ -520,6 +530,9 @@ class API(ModelView):
         is not read from the post parameters (where malicious user can tamper
         with them) but from the session.
 
+        .. versionadded:: 0.9.0
+           Added the `max_results_per_page` keyword argument.
+
         .. versionadded:: 0.7
            Added the `exclude_columns` keyword argument.
 
@@ -547,8 +560,7 @@ class API(ModelView):
             _parse_includes(include_columns)
         self.validation_exceptions = tuple(validation_exceptions or ())
         self.results_per_page = results_per_page
-        self.paginate = (isinstance(self.results_per_page, int)
-                         and self.results_per_page > 0)
+        self.max_results_per_page = max_results_per_page
         self.post_form_preprocessor = post_form_preprocessor
 
     def _add_to_relation(self, query, relationname, toadd=None):
@@ -574,14 +586,19 @@ class API(ModelView):
 
         """
         submodel = _get_related_model(self.model, relationname)
+        if isinstance(toadd, dict):
+            toadd = [toadd]
         for dictionary in toadd or []:
             if 'id' in dictionary:
                 subinst = self._get_by(dictionary['id'], submodel)
             else:
                 kw = unicode_keys_to_strings(dictionary)
                 subinst = _get_or_create(self.session, submodel, **kw)[0]
-            for instance in query:
-                getattr(instance, relationname).append(subinst)
+            try:
+                for instance in query:
+                    getattr(instance, relationname).append(subinst)
+            except AttributeError:
+                setattr(instance, relationname, subinst)
 
     def _remove_from_relation(self, query, relationname, toremove=None):
         """Removes a related model from each model specified by `query`.
@@ -622,17 +639,44 @@ class API(ModelView):
             if remove:
                 self.session.delete(subinst)
 
+    def _set_on_relation(self, query, relationname, toset=None):
+        """Sets the value of the relation specified by `relationname` on each
+        instance specified by `query` to have the new or existing related
+        models specified by `toset`.
+
+        This function does not commit the changes made to the database. The
+        calling function has that responsibility.
+
+        `query` is a SQLAlchemy query instance that evaluates to all instances
+        of the model specified in the constructor of this class that should be
+        updated.
+
+        `relationname` is the name of a one-to-many relationship which exists
+        on each model specified in `query`.
+
+        `toset` is a list of dictionaries, each representing the attributes of
+        an existing or new related model to set. If a dictionary contains the
+        key ``'id'``, that instance of the related model will be added.
+        Otherwise, the :classmethod:`~flask.ext.restless.model.get_or_create`
+        class method will be used to get or create a model to set.
+
+        """
+        submodel = _get_related_model(self.model, relationname)
+        subinst_list = []
+        for dictionary in toset or []:
+            if 'id' in dictionary:
+                subinst = self._get_by(dictionary['id'], submodel)
+            else:
+                kw = unicode_keys_to_strings(dictionary)
+                subinst = _get_or_create(self.session, submodel, **kw)[0]
+            subinst_list.append(subinst)
+        for instance in query:
+            setattr(instance, relationname, subinst_list)
+
     # TODO change this to have more sensible arguments
     def _update_relations(self, query, params):
-        """Adds or removes models which are related to the model specified in
-        the constructor of this class.
-
-        If one of the dictionaries specified in ``add`` or ``remove`` includes
-        an ``id`` key, the object with that ``id`` will be attempt to be added
-        or removed. Otherwise, an existing object with the specified attribute
-        values will be attempted to be added or removed. If adding, a new
-        object will be created if a matching object could not be found in the
-        database.
+        """Adds, removes, or sets models which are related to the model
+        specified in the constructor of this class.
 
         This function does not commit the changes made to the database. The
         calling function has that responsibility.
@@ -645,11 +689,20 @@ class API(ModelView):
         updated.
 
         `params` is a dictionary containing a mapping from name of the relation
-        to modify (as a string) to a second dictionary. The inner dictionary
-        contains at most two mappings, one with the key ``'add'`` and one with
-        the key ``'remove'``. Each of these is a mapping to a list of
-        dictionaries which represent the attributes of the object to add to or
-        remove from the relation.
+        to modify (as a string) to either a list or another dictionary. In the
+        former case, the relation will be assigned the instances specified by
+        the elements of the list, which are dictionaries as described below.
+        In the latter case, the inner dictionary contains at most two mappings,
+        one with the key ``'add'`` and one with the key ``'remove'``. Each of
+        these is a mapping to a list of dictionaries which represent the
+        attributes of the object to add to or remove from the relation.
+
+        If one of the dictionaries specified in ``add`` or ``remove`` (or the
+        list to be assigned) includes an ``id`` key, the object with that
+        ``id`` will be attempt to be added or removed. Otherwise, an existing
+        object with the specified attribute values will be attempted to be
+        added or removed. If adding, a new object will be created if a matching
+        object could not be found in the database.
 
         If a dictionary in one of the ``'remove'`` lists contains a mapping
         from ``'__delete__'`` to ``True``, then the removed object will be
@@ -660,10 +713,15 @@ class API(ModelView):
         relations = _get_relations(self.model)
         tochange = frozenset(relations) & frozenset(params)
         for columnname in tochange:
-            toadd = params[columnname].get('add', [])
-            toremove = params[columnname].get('remove', [])
-            self._add_to_relation(query, columnname, toadd=toadd)
-            self._remove_from_relation(query, columnname, toremove=toremove)
+            if isinstance(params[columnname], list):
+                toset = params[columnname]
+                self._set_on_relation(query, columnname, toset=toset)
+            else:
+                toadd = params[columnname].get('add', [])
+                toremove = params[columnname].get('remove', [])
+                self._add_to_relation(query, columnname, toadd=toadd)
+                self._remove_from_relation(query, columnname,
+                                           toremove=toremove)
         return tochange
 
     def _handle_validation_exception(self, exception):
@@ -699,8 +757,8 @@ class API(ModelView):
         # 'message' comes from savalidation
         if hasattr(exception, 'message'):
             # TODO this works only if there is one validation error
-            left, right = exception.message.rsplit(':', 1)
             try:
+                left, right = exception.message.rsplit(':', 1)
                 left_bracket = left.rindex('[')
                 right_bracket = right.rindex(']')
             except ValueError:
@@ -839,6 +897,21 @@ class API(ModelView):
                               include_relations=self.include_relations)
             return jsonify(result)
 
+    def _compute_results_per_page(self):
+        """Helper function which returns the number of results per page based
+        on the request argument ``results_per_page`` and the server
+        configuration parameters :attr:`results_per_page` and
+        :attr:`max_results_per_page`.
+
+        """
+        try:
+            results_per_page = int(request.args.get('results_per_page'))
+        except:
+            results_per_page = self.results_per_page
+        if results_per_page <= 0:
+            results_per_page = self.results_per_page
+        return min(results_per_page, self.max_results_per_page)
+
     # TODO it is ugly to have `deep` as an arg here; can we remove it?
     def _paginated(self, instances, deep):
         """Returns a paginated JSONified response from the specified list of
@@ -857,17 +930,19 @@ class API(ModelView):
            {
              "page": 2,
              "total_pages": 3,
+             "num_results": 8,
              "objects": [{"id": 1, "name": "Jeffrey", "age": 24}, ...]
            }
 
         """
         num_results = len(instances)
-        if self.paginate:
+        results_per_page = self._compute_results_per_page()
+        if results_per_page > 0:
             # get the page number (first page is page 1)
             page_num = int(request.args.get('page', 1))
-            start = (page_num - 1) * self.results_per_page
-            end = min(num_results, start + self.results_per_page)
-            total_pages = int(math.ceil(num_results / self.results_per_page))
+            start = (page_num - 1) * results_per_page
+            end = min(num_results, start + results_per_page)
+            total_pages = int(math.ceil(num_results / results_per_page))
         else:
             page_num = 1
             start = 0
@@ -991,7 +1066,12 @@ class API(ModelView):
             params = json.loads(request.data)
         except (TypeError, ValueError, OverflowError):
             return jsonify_status_code(400, message='Unable to decode data')
-
+        # Check for any request parameter naming a column which does not exist
+        # on the current model.
+        for field in params:
+            if not hasattr(self.model, field):
+                msg = "Model does not have field '%s'" % field
+                return jsonify_status_code(400, message=msg)
         # If post_form_preprocessor is specified, call it
         if self.post_form_preprocessor:
             params = self.post_form_preprocessor(params)
@@ -1059,14 +1139,19 @@ class API(ModelView):
 
         """
         self._check_authentication()
-
         # try to load the fields/values to update from the body of the request
         try:
             data = json.loads(request.data)
         except (TypeError, ValueError, OverflowError):
             # this also happens when request.data is empty
             return jsonify_status_code(400, message='Unable to decode data')
-
+        # Check for any request parameter naming a column which does not exist
+        # on the current model.
+        for field in data:
+            if not hasattr(self.model, field):
+                msg = "Model does not have field '%s'" % field
+                return jsonify_status_code(400, message=msg)
+        # Check if the request is to patch many instances of the current model.
         patchmany = instid is None
         if patchmany:
             try:
@@ -1078,6 +1163,8 @@ class API(ModelView):
         else:
             # create a SQLAlchemy Query which has exactly the specified row
             query = self._query_by_primary_key(instid)
+            if query.count() == 0:
+                abort(404)
             assert query.count() == 1, 'Multiple rows with same ID'
 
         relations = self._update_relations(query, data)
